@@ -12,13 +12,15 @@ import copy
 import functools
 import os
 
+import six
+
 from cloudinit import ec2_utils
 from cloudinit import log as logging
 from cloudinit import net
 from cloudinit import sources
-from cloudinit import subp
 from cloudinit import url_helper
 from cloudinit import util
+
 from cloudinit.sources import BrokenMetadata
 
 # See https://docs.openstack.org/user-guide/cli-config-drive.html
@@ -69,7 +71,6 @@ KNOWN_PHYSICAL_TYPES = (
     None,
     'bgpovs',  # not present in OpenStack upstream but used on OVH cloud.
     'bridge',
-    'cascading',  # not present in OpenStack upstream, used on OpenTelekomCloud
     'dvs',
     'ethernet',
     'hw_veb',
@@ -111,7 +112,7 @@ class SourceMixin(object):
             dev_entries = util.find_devs_with(criteria)
             if dev_entries:
                 device = dev_entries[0]
-        except subp.ProcessExecutionError:
+        except util.ProcessExecutionError:
             pass
         return device
 
@@ -162,7 +163,8 @@ class SourceMixin(object):
             return device
 
 
-class BaseReader(metaclass=abc.ABCMeta):
+@six.add_metaclass(abc.ABCMeta)
+class BaseReader(object):
 
     def __init__(self, base_path):
         self.base_path = base_path
@@ -225,7 +227,7 @@ class BaseReader(metaclass=abc.ABCMeta):
         """
 
         load_json_anytype = functools.partial(
-            util.load_json, root_types=(dict, list, str))
+            util.load_json, root_types=(dict, list) + six.string_types)
 
         def datafiles(version):
             files = {}
@@ -280,9 +282,8 @@ class BaseReader(metaclass=abc.ABCMeta):
                 try:
                     data = translator(data)
                 except Exception as e:
-                    raise BrokenMetadata(
-                        "Failed to process path %s: %s" % (path, e)
-                    ) from e
+                    raise BrokenMetadata("Failed to process "
+                                         "path %s: %s" % (path, e))
             if found:
                 results[name] = data
 
@@ -292,9 +293,8 @@ class BaseReader(metaclass=abc.ABCMeta):
             try:
                 metadata['random_seed'] = base64.b64decode(random_seed)
             except (ValueError, TypeError) as e:
-                raise BrokenMetadata(
-                    "Badly formatted metadata random_seed entry: %s" % e
-                ) from e
+                raise BrokenMetadata("Badly formatted metadata"
+                                     " random_seed entry: %s" % e)
 
         # load any files that were provided
         files = {}
@@ -306,9 +306,8 @@ class BaseReader(metaclass=abc.ABCMeta):
             try:
                 files[path] = self._read_content_path(item)
             except Exception as e:
-                raise BrokenMetadata(
-                    "Failed to read provided file %s: %s" % (path, e)
-                ) from e
+                raise BrokenMetadata("Failed to read provided "
+                                     "file %s: %s" % (path, e))
         results['files'] = files
 
         # The 'network_config' item in metadata is a content pointer
@@ -320,9 +319,8 @@ class BaseReader(metaclass=abc.ABCMeta):
                 content = self._read_content_path(net_item, decode=True)
                 results['network_config'] = content
             except IOError as e:
-                raise BrokenMetadata(
-                    "Failed to read network configuration: %s" % (e)
-                ) from e
+                raise BrokenMetadata("Failed to read network"
+                                     " configuration: %s" % (e))
 
         # To openstack, user can specify meta ('nova boot --meta=key=value')
         # and those will appear under metadata['meta'].
@@ -374,9 +372,8 @@ class ConfigDriveReader(BaseReader):
             try:
                 return util.load_json(self._path_read(path))
             except Exception as e:
-                raise BrokenMetadata(
-                    "Failed to process path %s: %s" % (path, e)
-                ) from e
+                raise BrokenMetadata("Failed to process "
+                                     "path %s: %s" % (path, e))
 
     def read_v1(self):
         """Reads a version 1 formatted location.
@@ -400,17 +397,13 @@ class ConfigDriveReader(BaseReader):
                 path = found[name]
                 try:
                     contents = self._path_read(path)
-                except IOError as e:
-                    raise BrokenMetadata("Failed to read: %s" % path) from e
+                except IOError:
+                    raise BrokenMetadata("Failed to read: %s" % path)
                 try:
-                    # Disable not-callable pylint check; pylint isn't able to
-                    # determine that every member of FILES_V1 has a callable in
-                    # the appropriate position
-                    md[key] = translator(contents)  # pylint: disable=E1102
+                    md[key] = translator(contents)
                 except Exception as e:
-                    raise BrokenMetadata(
-                        "Failed to process path %s: %s" % (path, e)
-                    ) from e
+                    raise BrokenMetadata("Failed to process "
+                                         "path %s: %s" % (path, e))
             else:
                 md[key] = copy.deepcopy(default)
 
@@ -421,11 +414,8 @@ class ConfigDriveReader(BaseReader):
         keydata = meta_js.get('public-keys', keydata)
         if keydata:
             lines = keydata.splitlines()
-            md['public-keys'] = [
-                line
-                for line in lines
-                if len(line) and not line.startswith("#")
-            ]
+            md['public-keys'] = [l for l in lines
+                                 if len(l) and not l.startswith("#")]
 
         # config-drive-v1 has no way for openstack to provide the instance-id
         # so we copy that into metadata from the user input
@@ -687,13 +677,11 @@ def convert_net_json(network_json=None, known_macs=None):
                 raise ValueError("Unable to find a system nic for %s" % d)
             d['name'] = known_macs[mac]
 
-        for cfg, key, fmt, targets in link_updates:
-            if isinstance(targets, (list, tuple)):
-                cfg[key] = [
-                    fmt % link_id_info[target]['name'] for target in targets
-                ]
+        for cfg, key, fmt, target in link_updates:
+            if isinstance(target, (list, tuple)):
+                cfg[key] = [fmt % link_id_info[l]['name'] for l in target]
             else:
-                cfg[key] = fmt % link_id_info[targets]['name']
+                cfg[key] = fmt % link_id_info[target]['name']
 
     # Infiniband interfaces may be referenced in network_data.json by a 6 byte
     # Ethernet MAC-style address, and we use that address to look up the
